/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * France Telecom Research and Development.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *	Author: Clayton Williams (claytonw@mit.edu)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsIDOMRemoteEventSource.h"
#include "nsIStreamListener.h"
#include "nsIRequest.h"
#include "nsIDOMRemoteEventTarget.h"
#include "nsIChannel.h"
#include "prlock.h"
#include "nsClassHashtable.h"
#include "nsIDOMDocument.h"
#include "nsIDOMDocumentEvent.h"
#include "nsIDOMDocumentView.h"
#include "nsIDOMEventTarget.h"


class nsDOMRemoteEventSource : public nsIDOMRemoteEventSource,
			       public nsIStreamListener
{
  NS_DECL_ISUPPORTS
  NS_DECL_NSIDOMREMOTEEVENTSOURCE
  NS_DECL_NSISTREAMLISTENER
  NS_DECL_NSIREQUESTOBSERVER
 public:
  nsDOMRemoteEventSource();

 private:
  ~nsDOMRemoteEventSource();

  nsresult GetBooleanValue(const nsCString* str, PRBool* aBool);
  nsresult GetIntegerValue(const nsCString* str, PRInt32* aInt32);
  nsresult GetInteger16Value(const nsCString* str, PRUint16* aUint16);

 protected:
  void parseStream(char* buffer, PRInt32 size, PRInt32 offset);
  void tryEvent();
  void fillDefaults();

  nsIDOMRemoteEventTarget* mTarget;

  PRBool initialized;

  nsString mSrc;

  nsIChannel* mChannel;

  PRLock* mLock;

  //These two must always be in sync
  char* mBuffer;
  PRInt32 mBufferSize;

  //the parser is a finite-state machine
  PRInt32 mState;
  PRInt32 mTokenStartPosition;
  PRInt32 mTokenSize;
  //mLastField is used in the fill-data state.
  //It is the name of the last field that was parsed,
  //which the data will fill into
  nsCString mLastField;

  //table of fields and values
  nsClassHashtable<nsCStringHashKey, nsCString> *mFieldTable;

  //reference to the DOM document, for getElementById
  nsIDOMDocument* mDoc;

  //reference to the DOM event factory interface
  nsIDOMDocumentEvent* mDocEventFactory;

  //reference to the view that UI objects will affect
  nsIDOMDocumentView* mDocView;

  //reference to the 'document' DOM object, which will be the target if a
  //remote event specifies "Document" as its target
  nsIDOMEventTarget* mDocTarget;

  //reference to the network request object which we keep so that we can
  //shut down the channel when this object is being destroyed
  nsIRequest*  mRequest;
};
